print("Starting chti116 v3.0")
#
#
#
# V3.0 : 2023.07.12 : custom v3.0 combo for æ and œ, capslock does capitalize É À Ê Ù Ë Ç etc
# 
# Last quirk to fix
# - pushing A+E at the same time is quite tricky... sometime e goes thru with alt and invoke menu shortcuts :( same for A+O.
#
# Features 
# - this script is for a special "chti116" key layout, but a lot could be used for a fullsize ISO keyboard
# - this is aimed at Windows operating system, with standard french AZERTY layout activated : can be used on pc you dont admin (i.e. at work)
# - It has a slighty changed AZERTY layout (ex: @ instead of ², key ^ is remapped one row higher, \ is ALTGR + :, ° is ALTGR+0, ² is ALTGR+2, ` is ALTGR+', etc.) 
# - ALTGR brings âêûîô on aeuio keys, ß on S, ~ on N, « and » on L and M, and more
# - Special CAPSLOCK behavior, when CAPSLOCK engaged, all éèàâêôç... are turned into CAPS, the rest is unchanged (just shifted)
# - pushing A+E or O+E put æ and œ but it's not very reliable... 
# - some media keys (play/pause, next, vol+, vol-, mute)
# - some shortcuts keys for duplicate (ctrl+D) cut, copy, paste, taskmanager, fileexplorer, CTRL+ALT+N and CTRL+ALT+C to program as shortcut to Notepad and Calculator or whatever (needs shortcut implementation on Windows side)
# - special FUNK key (just right to ALTGR) brings 
#    1- office shortcuts on F1-F12 launching outlook, teams, word, excel etc.
#    2- num, scrl lock, pause on specific numpad, insert on delete key, redo on specific undo key, € (on E)
#    3- FUNK+ESC reboot keyboard and if you maintain ESC if can skip boot.py (if configured that way) so I have usb drive mount only in this case.
#    4- FUNK+NUMPAD : 0=>standard layout for games, 1=>custom V1 (avoid), 2=>custom V2 (default), FUNK+NUMPADDOT=>types informations
#
#
#

import board

from kmk.kmk_keyboard import KMKKeyboard
from kmk.keys import KC
from kmk.scanners import DiodeOrientation

from kmk.extensions.lock_status import LockStatus
from kmk.extensions.stringy_keymaps import StringyKeymaps
from kmk.modules.layers import Layers
from kmk.handlers.sequences import simple_key_sequence
from kmk.extensions.media_keys import MediaKeys
#from kmk.modules.modtap import ModTap
from kmk.extensions.international import International
from kmk.consts import UnicodeMode
#from kmk.handlers.sequences import unicode_string_sequence
#from kmk.handlers.sequences import unicode_codepoint_sequence
from kmk.handlers.sequences import send_string
from kmk.keys import make_key
from kmk.modules.combos import Combos, Chord, Sequence
from kmk.types import AttrDict, KeySequenceMeta


combos = Combos()
lck = LockStatus()


chti116 = KMKKeyboard()
chti116.modules.append(combos)
chti116.extensions.append(lck)


chti116.col_pins = (board.GP0, board.GP1, board.GP2, board.GP3, board.GP4, board.GP5, board.GP6, board.GP7, board.GP8, board.GP9, board.GP10, board.GP11, board.GP12, board.GP13, board.GP14, board.GP16, board.GP28, board.GP17, )
chti116.row_pins = (board.GP18, board.GP19, board.GP20, board.GP21, board.GP22, board.GP26, board.GP27, )
chti116.diode_orientation = DiodeOrientation.COL2ROW

media = MediaKeys() 

chti116.modules.append(Layers())
chti116.extensions.append(MediaKeys())
#chti116.modules.append(modtap) 
chti116.extensions.append(International())
#chti116.unicode_mode = UnicodeMode.RALT # DOESNT WORK FOR NOW WORKAROUND USED

# this for better looking formatting...
______ = KC.NO	 # no physical key there
_____v = KC.TRNS # same key as the previous layer (from where you came)




# My keyboard have 3 modes : 
#   - standard layout                  ==> to use with games,  LAYOUTS : 1 =[FUNK]=> 2
#   - custom 1 (very modded AZERTY)    ==> avoid it,           LAYOUTS : 6 =[SHIFT]=> 7; 6 =[ALTGR]=> 8; 6 =[FUNK]=> 9
#   - custom 2 (lightly modded AZERTY) ==> that's the default  LAYOUTS : 0 =[SHIFT]=> 4; 0 =[ALTGR]=> 5; 0 =[FUNK]=> 3

# for standard mode : just 1 other layer with special FUNK key (quite funky)
FUNKS = KC.MO(2)

# for custom 1 : 3 other layers
SFTC =  KC.LM(7, KC.LSFT) # maintain shift to avoid shift releasing  
ALTC =  KC.MO(8)
FUNKC = KC.MO(9)

# for custom 2 : 4 others layers
SFTC2 = KC.LM(4, KC.LSFT) # maintain shift to avoid shift releasing
ALTC2 = KC.MO(5)
FUNKC2 = KC.MO(3)


# switching mode
GOSTD = KC.TO(1)
GOCUS1 = KC.TO(6)
GOCUS2 = KC.TO(0)

# Types the current status of the keyboard and version
INFOKC2 = simple_key_sequence(( KC.C, KC.U, KC.S, KC.T, KC.O, KC.SCLN, KC.P2, KC.SPC, KC.P3, KC.PDOT, KC.P0,  ))
INFOKC1 = simple_key_sequence(( KC.C, KC.U, KC.S, KC.T, KC.O, KC.SCLN, KC.P1, KC.SPC, KC.P3, KC.PDOT, KC.P0,  ))
INFOKS = simple_key_sequence(( KC.S, KC.T, KC.Q, KC.N, KC.D, KC.Q, KC.R, KC.D, KC.SPC, KC.P3, KC.PDOT, KC.P0,  ))


# Some combo to type special characters on an AZERTY layout
AROB = KC.RALT(KC.N0)    # arobase "@"
PCT = KC.LSFT(KC.QUOT)   # percent "%"
CHAPO = KC.LBRC          # circumflex dead key "^"
GRAVE = KC.RALT(KC.N7)   # grave accent dead key "`"
GRAVE2 = KC.N7           # grave accent dead key when already RALTed
UGRAVE = KC.QUOT     	 # u with grave accent "ù"
PARO = KC.N5             # parenthese open "("
PARF = KC.MINS           # parenthese close ")"
MOINS = KC.N6			 # minus "-"
APOST = KC.N4            # apostrophe ' (simple quote aka guillemet simple, ...)
CROCO = simple_key_sequence(( KC.LSFT(no_press=True), KC.RALT(KC.N5), ))   # [when already shifted] crochet open "["
CROCF = simple_key_sequence(( KC.LSFT(no_press=True), KC.RALT(KC.MINS), )) # [when already shifted] crochet close "]"
GUILL = simple_key_sequence(( KC.LSFT(no_press=True), KC.N3, ))            # [when already shifted] guillemets " (double quote, guillemet double, ...)
DIESE = simple_key_sequence(( KC.LSFT(no_press=True), KC.RALT(KC.N3), ))   # [when already shifted] hyphen #
SOULI = simple_key_sequence(( KC.LSFT(no_press=True), KC.N8, ))            # [when already shifted] underscore (blanc souligne) _
ACCOO = simple_key_sequence(( KC.LSFT(no_press=True), KC.RALT(KC.N4), ))   # [when already shifted] accolade open "{"
ACCOF = simple_key_sequence(( KC.LSFT(no_press=True), KC.RALT(KC.EQL), ))  # [when already shifted] accolade close "}"
DOLLAR = simple_key_sequence(( KC.LSFT(no_press=True), KC.RBRC, ))         # [when already shifted] dollar $
PIPE = simple_key_sequence(( KC.LSFT(no_press=True), KC.RALT(KC.N6), ))    # [when already shifted] pipe aka barre verticale "|"
CARRE = KC.GRV           # square (little 2) "²"
MU = KC.LSFT(KC.NUHS)    # sort of u starting with a raising bar "µ"
TILDE = KC.RALT(KC.N2)   # tilda, kind of wave "~"
DEGRE = KC.LSFT(KC.MINS) # degree symbol (very little zero) "°"
BACKSL = KC.RALT(KC.N8)  # backslash aka barre oblique descendante "\"
BACKSL2 = simple_key_sequence(( KC.LSFT(no_press=True), KC.RALT(KC.N8), )) # [when already shifted] backslash 
PARAG = KC.LSFT(KC.SLSH) # paragraph symbol, like two S overlapping "§"
LIVRE = KC.LSFT(KC.RBRC) # pound aka livre sterling "£"
EURO = KC.RALT(KC.E)     # euro currency symbol "€"
CEDIL = KC.N9            # c cedille "ç"

# vowels with trema and circumflex
ATREMA = simple_key_sequence((KC.LSFT(KC.LBRC),KC.Q),) # a trema : ä
ETREMA = simple_key_sequence((KC.LSFT(KC.LBRC),KC.E),) # e trema : ë
ITREMA = simple_key_sequence((KC.LSFT(KC.LBRC),KC.I),) # i trema : ï
OTREMA = simple_key_sequence((KC.LSFT(KC.LBRC),KC.O),) # o trema : ö
UTREMA = simple_key_sequence((KC.LSFT(KC.LBRC),KC.U),) # u trema : ü

ACIRC = simple_key_sequence((KC.LBRC,KC.Q),)           # a circumflex : â
ECIRC = simple_key_sequence((KC.LBRC,KC.E),)           # e circumflex : ê
ICIRC = simple_key_sequence((KC.LBRC,KC.I),)           # i circumflex : î
OCIRC = simple_key_sequence((KC.LBRC,KC.O),)           # o circumflex : ô
UCIRC = simple_key_sequence((KC.LBRC,KC.U),)           # u circumflex : û

# other official yet unknown french symbols
GUIFO = simple_key_sequence( ( KC.LALT(no_release=True), KC.P0, KC.P1, KC.P7, KC.P1 , KC.LALT(no_press=True), ) ) # Official french opening quotes «
GUIFF = simple_key_sequence( ( KC.LALT(no_release=True), KC.P0, KC.P1, KC.P8, KC.P7 , KC.LALT(no_press=True), ) ) # Official french closing quotes »
CADRA = simple_key_sequence( ( KC.LALT(no_release=True), KC.P0, KC.P1, KC.P5, KC.P1 , KC.LALT(no_press=True), ) ) # Long dash aka Tiret cadratin "—"
ESZETT = simple_key_sequence( ( KC.LALT(no_release=True), KC.P0, KC.P2, KC.P2, KC.P3 , KC.LALT(no_press=True), ) )# looking like beta but german symbol for ss "ß"

CAPECUTE = simple_key_sequence( ( KC.LALT(no_release=True), KC.P1, KC.P4, KC.P4, KC.LALT(no_press=True), ) )      # é but in caps : É
CAPEGRAVE = simple_key_sequence( ( KC.LALT(no_release=True), KC.P2, KC.P1, KC.P2, KC.LALT(no_press=True), ) )     # è but in caps: È
CAPAGRAVE = simple_key_sequence( ( KC.LALT(no_release=True), KC.P1, KC.P8, KC.P3, KC.LALT(no_press=True), ) )     # à but in caps: À
CAPCEDIL = simple_key_sequence( ( KC.LALT(no_release=True), KC.P1, KC.P2, KC.P8, KC.LALT(no_press=True), ) )      # ç but in caps : Ç

UNIAE = simple_key_sequence( ( KC.LALT(no_release=True), KC.P1, KC.P4, KC.P5, KC.LALT(no_press=True), ) )         # a and e in one char : "æ"
UNIOE = simple_key_sequence( ( KC.LALT(no_release=True), KC.P1, KC.P5, KC.P6, KC.LALT(no_press=True), ) )         # o and e in one char : "œ"

CURSIG = KC.RALT(KC.RBRC)# Currency sign ou currency placeholder aka scarab, turtle, small round with 4 feet U+00A4 ou ALT+0164 : "¤"



# this function is like a simple_key_sequence but called on the fly by a function : usefull for make_key
def pushCodeList(keylist):
	for k in keylist:
		chti116.process_key(k, is_pressed=True)
		chti116._send_hid()
		chti116.process_key(k, is_pressed=False)
		chti116._send_hid()		

	
# this function is like a simple_key_sequence but specific for ALT+code. It's called on the fly by a function : usefull for make_key	
def pushAltedCodeList(keylist):
	chti116.process_key(KC.LALT, is_pressed=True)
	chti116._send_hid()
	for k in keylist:
		chti116.process_key(k, is_pressed=True)
		chti116._send_hid()
		chti116.process_key(k, is_pressed=False)
		chti116._send_hid()		

	chti116.process_key(KC.LALT, is_pressed=False)
	chti116._send_hid()
	
# deprecated : doesnt work well : the keys are nearly all released at same time... could cause problems
def DEPRECATEDpushAltedCodeList(keylist):
	chti116.add_key(KC.LALT)
	chti116._send_hid()
	for k in keylist:
		chti116.tap_key(k)
		chti116._send_hid()	

	chti116.remove_key(KC.LALT)
	chti116._send_hid()

	


# BUGGY : how to push predefined 'simple_key_sequence' from here ? kepts for further tests
# Special letters with 2 behaviors depending on capslock status
def BUGmyecute(*args, **kwargs):
	if lck.get_caps_lock():
		### I just need to push the CAPECUTE sequence here, but how ??
		
		# chti116.tap_key(CAPECUTE)                       # doesnt work, maybe because CAPECUTE is a sequence
		
		# chti116.process_key(CAPECUTE, is_pressed=True)  # doesnt work, maybe because CAPECUTE is a sequence
		# chti116._send_hid()
		
		# CAPECUTE()                                      # doesnt work, CAPECUTE isnt a function
		chti116.tap_key(KC.N8)
	else:
		chti116.tap_key(KC.N2) # works fine
		

# DEPRECATED : a heavy way of coding one key with 2 behaviours depending on capslock status. Replaced with lambda function, see further down		
# def myecute(*args, **kwargs):
	# if lck.get_caps_lock():
		# pushAltedCodeList([KC.P1, KC.P4, KC.P4])
	# else:
		# chti116.tap_key(KC.N2) # works fine
#make_key(names=('KECUTE',), on_press=myecute)


# All those keys have 2 behaviours depending on capslock status,
# The aim of this is to get a special capslock behaviour for special keys like é, à, ù etc. that is not just shifted keys
# 
#  I'm making new KC.KECUTE key, when pressed does > HERE WHEN CAPSLOCK IS ACTIVATED, i.e. push "É" <======= else =======> PUSH THE NORMAL KEY i.e. "é" <
make_key(names=('KECUTE',),  on_press=lambda *args: pushAltedCodeList([KC.P1, KC.P4, KC.P4])        if lck.get_caps_lock() else chti116.tap_key(KC.N2))
make_key(names=('KEGRAVE',), on_press=lambda *args: pushAltedCodeList([KC.P2, KC.P1, KC.P2])        if lck.get_caps_lock() else chti116.tap_key(KC.N7))
make_key(names=('KAGRAVE',), on_press=lambda *args: pushAltedCodeList([KC.P1, KC.P8, KC.P3])        if lck.get_caps_lock() else chti116.tap_key(KC.N0))
make_key(names=('KUGRAVE',), on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P1, KC.P7]) if lck.get_caps_lock() else chti116.tap_key(KC.QUOT))
make_key(names=('KCEDIL',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P1, KC.P9, KC.P9]) if lck.get_caps_lock() else chti116.tap_key(KC.N9))

make_key(names=('KACIRC',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P1, KC.P9, KC.P4]) if lck.get_caps_lock() else pushCodeList([KC.LBRC,KC.Q]))
make_key(names=('KECIRC',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P0, KC.P2]) if lck.get_caps_lock() else pushCodeList([KC.LBRC,KC.E]))
make_key(names=('KICIRC',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P0, KC.P6]) if lck.get_caps_lock() else pushCodeList([KC.LBRC,KC.I]))
make_key(names=('KOCIRC',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P1, KC.P2]) if lck.get_caps_lock() else pushCodeList([KC.LBRC,KC.O]))
make_key(names=('KUCIRC',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P1, KC.P9]) if lck.get_caps_lock() else pushCodeList([KC.LBRC,KC.U]))

make_key(names=('KATREM',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P1, KC.P9, KC.P6]) if lck.get_caps_lock() else pushCodeList([KC.LSFT(KC.LBRC),KC.Q]))
make_key(names=('KETREM',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P0, KC.P3]) if lck.get_caps_lock() else pushCodeList([KC.LSFT(KC.LBRC),KC.E]))
make_key(names=('KITREM',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P0, KC.P7]) if lck.get_caps_lock() else pushCodeList([KC.LSFT(KC.LBRC),KC.I]))
make_key(names=('KOTREM',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P1, KC.P4]) if lck.get_caps_lock() else pushCodeList([KC.LSFT(KC.LBRC),KC.O]))
make_key(names=('KUTREM',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P2, KC.P2, KC.P0]) if lck.get_caps_lock() else pushCodeList([KC.LSFT(KC.LBRC),KC.U]))

make_key(names=('KUNIAE',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P1, KC.P9, KC.P8]) if lck.get_caps_lock() else pushAltedCodeList([KC.P1, KC.P4, KC.P5]))
make_key(names=('KUNIOE',),  on_press=lambda *args: pushAltedCodeList([KC.P0, KC.P1, KC.P4, KC.P0]) if lck.get_caps_lock() else pushAltedCodeList([KC.P0, KC.P1, KC.P5, KC.P6]))


# Windows shortcuts provided by the use of the "office key"
WORD = KC.HYPR(KC.Z)  # the infamous office key + W = launch microsoft Word (on azerty, had to replace to Z)
EXCEL = KC.HYPR(KC.X) # the infamous office key + X = launch microsoft Excel
POWPNT = KC.HYPR(KC.P)# the infamous office key + P = launch microsoft Powerpoint
OUTLK = KC.HYPR(KC.O) # the infamous office key + O = launch microsoft Outlook
ODRIVE = KC.HYPR(KC.D)# the infamous office key + D = launch microsoft OneDrive
ONOTE = KC.HYPR(KC.N) # the infamous office key + N = launch microsoft OneNote
LNKIN = KC.HYPR(KC.L) # the infamous office key + L = launch LinkedIn web site
TEAMS = KC.HYPR(KC.T) # the infamous office key + T = launch microsoft teams
YAMMR = KC.HYPR(KC.Y) # the infamous office key + Y = launch Yammer web site


# standard combo for edit operations
DUP = KC.LCTL(KC.D)  # CTRL D works as duplicate in some software, not very standard though
CUT = KC.LCTL(KC.X)
COPY = KC.LCTL(KC.C)
PASTE = KC.LCTL(KC.V)
UNDO = KC.LCTL(KC.W) # CTRL Z but on a AZERTY
REDO = simple_key_sequence( ( KC.LSFT(no_press=True), KC.LCTL(KC.Y), ) ) # CTRL Y but if called with a SHIFT, I disable the shift first

# some combo to launch some things (in windows) 
TASKS = KC.LCTL(KC.LSFT(KC.ESC))
XPLR = KC.LGUI(KC.E)
LOCK = KC.LGUI(KC.L)
NOTE = KC.LCTL(KC.LALT(KC.N)) # pre-requisite : define a shortcut yourself on the windows side : edit the shortcut then press the key [CTRL+ALT+N]
CALC = KC.LCTL(KC.LALT(KC.C)) # pre-requisite : define a shortcut yourself on the windows side : edit the shortcut then press the key [CTRL+ALT+C]
EMOJI = KC.LGUI(KC.COMM)
PARAM = KC.LGUI(KC.I) # windows params




chti116.keymap = [
    [#  0:AZERTY CUSTOM v2 // default is an lightly modded AZERTY
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        KC.ESC,   KC.F1,    KC.F2,    KC.F3,    KC.F4,    KC.F5,    KC.F6,    KC.F7,    KC.F8,    KC.F9,   KC.F10,   KC.F11,   KC.F12,  KC.PSCR,     PARO,     PARF,  KC.PSLS,   KC.PAST,  
        AROB,     KC.N1,KC.KECUTE,   KC.N3,    KC.N4,    KC.N5,     KC.N6,KC.KEGRAVE,  KC.N8,    KC.N9,KC.KAGRAVE, KC.LBRC,   KC.EQL,  KC.BSPC,    KC.P7,    KC.P8,    KC.P9,   KC.PMNS,  
        KC.TAB,    KC.Q,     KC.W,     KC.E,     KC.R,     KC.T,     KC.Y,     KC.U,     KC.I,     KC.O,     KC.P,    CROCO,    CROCF, KC.ENTER,    KC.P4,    KC.P5,    KC.P6,   KC.PPLS,  
        KC.CAPS,   KC.A,     KC.S,     KC.D,     KC.F,     KC.G,     KC.H,     KC.J,     KC.K,     KC.L,  KC.SCLN,KC.KUGRAVE, KC.NUHS,   ______,    KC.P1,    KC.P2,    KC.P3,   KC.PENT,  
        SFTC2,  KC.NUBS,     KC.Z,     KC.X,     KC.C,     KC.V,     KC.B,     KC.N,     KC.M,  KC.COMM,   KC.DOT,  KC.SLSH,    SFTC2,   ______,    KC.P0,   ______,  KC.PDOT,    ______,  
        KC.LCTL,KC.LGUI,   ______,  KC.LALT,   ______,   ______,   KC.SPC,   ______,   ______,   ______,    ALTC2,   FUNKC2,  KC.RCTL,     UNDO,  KC.HOME,    KC.UP,   KC.END,   KC.PGUP,  
        TASKS,     XPLR,     NOTE,     CALC,      DUP,      CUT,     COPY,    PASTE,  KC.MPLY,  KC.MNXT,  KC.MUTE,  KC.VOLD,  KC.VOLU,   KC.DEL,  KC.LEFT,  KC.DOWN, KC.RIGHT,   KC.PGDN,  
    ],
    [#  1:AZERTY/QWERTY STANDARD
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        KC.ESC,   KC.F1,    KC.F2,    KC.F3,    KC.F4,    KC.F5,    KC.F6,    KC.F7,    KC.F8,    KC.F9,   KC.F10,   KC.F11,   KC.F12,  KC.PSCR,  KC.LBRC,  KC.RBRC,  KC.PSLS,   KC.PAST,  
        KC.GRV,   KC.N1,    KC.N2,    KC.N3,    KC.N4,    KC.N5,    KC.N6,    KC.N7,    KC.N8,    KC.N9,    KC.N0,  KC.MINS,   KC.EQL,  KC.BSPC,    KC.P7,    KC.P8,    KC.P9,   KC.PMNS,  
        KC.TAB,    KC.Q,     KC.W,     KC.E,     KC.R,     KC.T,     KC.Y,     KC.U,     KC.I,     KC.O,     KC.P,  KC.LBRC,  KC.RBRC, KC.ENTER,    KC.P4,    KC.P5,    KC.P6,   KC.PPLS,  
        KC.CAPS,   KC.A,     KC.S,     KC.D,     KC.F,     KC.G,     KC.H,     KC.J,     KC.K,     KC.L,  KC.SCLN,  KC.QUOT,  KC.NUHS,   ______,    KC.P1,    KC.P2,    KC.P3,   KC.PENT,  
        KC.LSFT,KC.NUBS,     KC.Z,     KC.X,     KC.C,     KC.V,     KC.B,     KC.N,     KC.M,  KC.COMM,   KC.DOT,  KC.SLSH,  KC.RSFT,   ______,    KC.P0,   ______,  KC.PDOT,    ______,  
        KC.LCTL,KC.LGUI,   ______,  KC.LALT,   ______,   ______,   KC.SPC,   ______,   ______,   ______,  KC.RALT,    FUNKS,  KC.RCTL,     UNDO,  KC.HOME,    KC.UP,   KC.END,   KC.PGUP,  
        TASKS,     XPLR,     NOTE,     CALC,      DUP,      CUT,     COPY,    PASTE,  KC.MPLY,  KC.MNXT,  KC.MUTE,  KC.VOLD,  KC.VOLU,   KC.DEL,  KC.LEFT,  KC.DOWN, KC.RIGHT,   KC.PGDN,  
    ],
    [#  2:STANDARD + FUNK 
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        _____v,   OUTLK,    TEAMS,     WORD,    EXCEL,   POWPNT,   _____v,   _____v,    ONOTE,   ODRIVE,     YAMMR,  LNKIN,     EMOJI,   _____v,  KC.NLCK,  KC.SLCK,  KC.PAUS,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,    _____v,
        _____v,  _____v,   _____v,     EURO,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   ______,   GOCUS1,   GOCUS2,   _____v,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   ______,    GOSTD,   ______,   INFOKS,    ______,
        _____v,  _____v,   ______,   _____v,   ______,   ______,   KC.SPC,   ______,   ______,   ______,  KC.RALT,  KC.RGUI,  KC.RCTL,     REDO,   _____v,   _____v,   _____v,    _____v, 
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   KC.INS,   _____v,   _____v,   _____v,    _____v,
    ],
    [#  3:CUSTOM v2 + FUNK 
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        KC.RESET, OUTLK,    TEAMS,     WORD,    EXCEL,   POWPNT,   CAPECUTE,   _____v,    ONOTE,   ODRIVE,     YAMMR,  LNKIN,     EMOJI,   _____v,  KC.NLCK,  KC.SLCK,  KC.PAUS,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,    _____v,
        _____v,  _____v,   _____v,     EURO,   _____v,   _____v,   _____v,       MU,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   ______,   GOCUS1,   GOCUS2,   _____v,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   ______,    GOSTD,   ______,  INFOKC2,    ______,
        _____v,  _____v,   ______,   _____v,   ______,   ______,   KC.SPC,   ______,   ______,   ______,  KC.RALT,  KC.RGUI,  KC.RCTL,     REDO,   _____v,   _____v,   _____v,    _____v, 
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   KC.INS,   _____v,   _____v,   _____v,    _____v,
    ],
    [#  4:AZERTY CUSTOM v2 + SHIFT. 
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        KC.ESC,   KC.F1,    KC.F2,    KC.F3,    KC.F4,    KC.F5,    KC.F6,    KC.F7,    KC.F8,    KC.F9,   KC.F10,   KC.F11,   KC.F12,  KC.PSCR,    CROCO,    CROCF,  BACKSL2,   KC.PAST,  
        DIESE,    KC.N1,    KC.N2,    KC.N3,    KC.N4,    KC.N5,    KC.N6,    KC.N7,    KC.N8,    KC.N9,    KC.N0,  KC.LBRC,   KC.EQL,  KC.BSPC,    KC.P7,    KC.P8,    KC.P9,   KC.PMNS,  
        KC.TAB,    KC.Q,     KC.W,     KC.E,     KC.R,     KC.T,     KC.Y,     KC.U,     KC.I,     KC.O,     KC.P,    ACCOO,    ACCOF, KC.ENTER,    KC.P4,    KC.P5,    KC.P6,   KC.PPLS,  
        KC.CAPS,   KC.A,     KC.S,     KC.D,     KC.F,     KC.G,     KC.H,     KC.J,     KC.K,     KC.L,  KC.SCLN,  KC.QUOT,   DOLLAR,   ______,    KC.P1,    KC.P2,    KC.P3,   KC.PENT,  
        KC.LSFT,KC.NUBS,     KC.Z,     KC.X,     KC.C,     KC.V,     KC.B,     KC.N,     KC.M,  KC.COMM,   KC.DOT,     PIPE,  KC.RSFT,   ______,    KC.P0,   ______,  KC.PDOT,    ______,  
        KC.LCTL,KC.LGUI,   ______,  KC.LALT,   ______,   ______,   KC.SPC,   ______,   ______,   ______,  KC.RALT,   FUNKC2,  KC.RCTL,     REDO,  KC.HOME,    KC.UP,   KC.END,   KC.PGUP,  
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,  KC.MPRV,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,    _____v,  
    ],    
    [#  5:AZERTY CUSTOM v2 + ALTGR 
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        KC.ESC,   KC.F1,    KC.F2,    KC.F3,    KC.F4,    KC.F5,    KC.F6,    KC.F7,    KC.F8,    KC.F9,   KC.F10,   KC.F11,   KC.F12,  KC.PSCR,  KC.LBRC,  KC.RBRC,  KC.PSLS,   KC.PAST,  
        CURSIG,KC.KATREM,   CARRE,KC.KETREM,    GRAVE,     PARF,    CADRA,KC.KUTREM,KC.KITREM,KC.KOTREM,    DEGRE,     PARO,     PARF,  KC.BSPC,    KC.P7,    KC.P8,    KC.P9,   KC.PMNS,  
        KC.TAB,KC.KACIRC,    KC.W,KC.KECIRC,     KC.R,     KC.T,     KC.Y,KC.KUCIRC,KC.KICIRC,KC.KOCIRC,  PCT, KC.NUBS, KC.LSFT(KC.NUBS), KC.ENTER, KC.P4,    KC.P5,    KC.P6,   KC.PPLS,  
        KC.CAPS,   KC.A,   ESZETT,     KC.D,     KC.F,     KC.G,     KC.H,     KC.J,     KC.K,    GUIFO,    GUIFF,  KC.QUOT,    LIVRE,   ______,    KC.P1,    KC.P2,    KC.P3,   KC.PENT,  
        KC.LSFT,KC.NUBS,     KC.Z,     KC.X,KC.KCEDIL,     KC.V,   ESZETT,    TILDE,     KC.M,  KC.COMM,   BACKSL,    PARAG,  KC.RSFT,   ______,    KC.P0,   ______,  KC.PDOT,    ______,  
        KC.LCTL,KC.LGUI,   ______,  KC.LALT,   ______,   ______,   KC.SPC,   ______,   ______,   ______,  KC.RALT,   FUNKC2,  KC.RCTL,     UNDO,  KC.HOME,    KC.UP,   KC.END,   KC.PGUP,  
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,    _____v,
    ],
    [#  6:AZERTY CUSTOM v1
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        KC.ESC,   KC.F1,    KC.F2,    KC.F3,    KC.F4,    KC.F5,    KC.F6,    KC.F7,    KC.F8,    KC.F9,   KC.F10,   KC.F11,   KC.F12,  KC.PSCR,     PARO,     PARF,  KC.PSLS,   KC.PAST,  
        AROB,     KC.N1,    KC.N0,    KC.N2,    KC.N7,      PCT,    CHAPO,    GRAVE,   UGRAVE,     PARO,     PARF,    MOINS,   KC.EQL,  KC.BSPC,    KC.P7,    KC.P8,    KC.P9,   KC.PMNS,  
        KC.TAB,    KC.Q,     KC.W,     KC.E,     KC.R,     KC.T,     KC.Y,     KC.U,     KC.I,     KC.O,     KC.P,    CROCO,    CROCF, KC.ENTER,    KC.P4,    KC.P5,    KC.P6,   KC.PPLS,  
        KC.CAPS,   KC.A,     KC.S,     KC.D,     KC.F,     KC.G,     KC.H,     KC.J,     KC.K,     KC.L,  KC.SCLN,    APOST,  KC.NUHS,   ______,    KC.P1,    KC.P2,    KC.P3,   KC.PENT,  
        SFTC,   KC.NUBS,     KC.Z,     KC.X,     KC.C,     KC.V,     KC.B,     KC.N,     KC.M,  KC.COMM,   KC.DOT,  KC.SLSH,     SFTC,   ______,    KC.P0,   ______,  KC.PDOT,    ______,  
        KC.LCTL,KC.LGUI,   ______,  KC.LALT,   ______,   ______,   KC.SPC,   ______,   ______,   ______,     ALTC,    FUNKC,  KC.RCTL,     UNDO,  KC.HOME,    KC.UP,   KC.END,   KC.PGUP,  
        TASKS,     XPLR,     NOTE,     CALC,      DUP,      CUT,     COPY,    PASTE,  KC.MPLY,  KC.MNXT,  KC.MUTE,  KC.VOLD,  KC.VOLU,   KC.DEL,  KC.LEFT,  KC.DOWN, KC.RIGHT,   KC.PGDN,  
    ],    
    [#  7:AZERTY CUSTOM v1 + SHIFT. 
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        KC.ESC,   KC.F1,    KC.F2,    KC.F3,    KC.F4,    KC.F5,    KC.F6,    KC.F7,    KC.F8,    KC.F9,   KC.F10,   KC.F11,   KC.F12,  KC.PSCR,    CROCO,    CROCF,  KC.PSLS,   KC.PAST,  
        DIESE,    KC.N1,    KC.N2,    KC.N3,    KC.N4,    KC.N5,    KC.N6,    KC.N7,    KC.N8,    KC.N9,    KC.N0,    SOULI,   KC.EQL,  KC.BSPC,    KC.P7,    KC.P8,    KC.P9,   KC.PMNS,  
        KC.TAB,    KC.Q,     KC.W,     KC.E,     KC.R,     KC.T,     KC.Y,     KC.U,     KC.I,     KC.O,     KC.P,    ACCOO,    ACCOF, KC.ENTER,    KC.P4,    KC.P5,    KC.P6,   KC.PPLS,  
        KC.CAPS,   KC.A,     KC.S,     KC.D,     KC.F,     KC.G,     KC.H,     KC.J,     KC.K,     KC.L,  KC.SCLN,    GUILL,   DOLLAR,   ______,    KC.P1,    KC.P2,    KC.P3,   KC.PENT,  
        KC.LSFT,KC.NUBS,     KC.Z,     KC.X,     KC.C,     KC.V,     KC.B,     KC.N,     KC.M,  KC.COMM,   KC.DOT,     PIPE,  KC.RSFT,   ______,    KC.P0,   ______,  KC.PDOT,    ______,  
        KC.LCTL,KC.LGUI,   ______,  KC.LALT,   ______,   ______,   KC.SPC,   ______,   ______,   ______,  KC.RALT,    FUNKC,  KC.RCTL,     REDO,  KC.HOME,    KC.UP,   KC.END,   KC.PGUP,  
        TASKS,     XPLR,     NOTE,     CALC,      DUP,      CUT,     COPY,    PASTE,  KC.MPLY,  KC.MPRV,  KC.MUTE,  KC.VOLD,  KC.VOLU,   KC.DEL,  KC.LEFT,  KC.DOWN, KC.RIGHT,   KC.PGDN,  
    ],    
    [#  8:AZERTY CUSTOM v1 + ALTGR 
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        KC.ESC,   KC.F1,    KC.F2,    KC.F3,    KC.F4,    KC.F5,    KC.F6,    KC.F7,    KC.F8,    KC.F9,   KC.F10,   KC.F11,   KC.F12,  KC.PSCR,  KC.LBRC,  KC.RBRC,  KC.PSLS,   KC.PAST,  
        CURSIG,  ATREMA,    CARRE,   ETREMA,    ECIRC,       MU,    TILDE,   UTREMA,   ITREMA,   OTREMA,    DEGRE,    CADRA,   KC.EQL,  KC.BSPC,    KC.P7,    KC.P8,    KC.P9,   KC.PMNS,  
        KC.TAB,   ACIRC,     KC.W,     EURO,     KC.R,     KC.T,     KC.Y,    UCIRC,    ICIRC,    OCIRC,  PCT, KC.NUBS, KC.LSFT(KC.NUBS), KC.ENTER, KC.P4,    KC.P5,    KC.P6,   KC.PPLS,  
        KC.CAPS,   KC.A,   ESZETT,     KC.D,     KC.F,     KC.G,     KC.H,     KC.J,     KC.K,    GUIFO,    GUIFF,  KC.QUOT,    LIVRE,   ______,    KC.P1,    KC.P2,    KC.P3,   KC.PENT,  
        KC.LSFT,KC.NUBS,     KC.Z,     KC.X,    CEDIL,     KC.V,   ESZETT,    TILDE,     KC.M,  KC.COMM,   BACKSL,    PARAG,  KC.RSFT,   ______,    KC.P0,   ______,  KC.PDOT,    ______,  
        KC.LCTL,KC.LGUI,   ______,  KC.LALT,   ______,   ______,   KC.SPC,   ______,   ______,   ______,  KC.RALT,    FUNKC,  KC.RCTL,     UNDO,  KC.HOME,    KC.UP,   KC.END,   KC.PGUP,  
        TASKS,     XPLR,     NOTE,     CALC,      DUP,      CUT,     COPY,    PASTE,  KC.MPLY,  KC.MNXT,  KC.MUTE,  KC.VOLD,  KC.VOLU,   KC.DEL,  KC.LEFT,  KC.DOWN, KC.RIGHT,   KC.PGDN,  
    ],
    [#  9:AZERTY CUSTOM v1 + FUNK 
     #   0          1         2          3        4         5         6         7          8        9        10         11       12        13       14         15        16        17
        _____v,   OUTLK,    TEAMS,     WORD,    EXCEL,   POWPNT,   _____v,   _____v,    ONOTE,   ODRIVE,     YAMMR,  LNKIN,     EMOJI,   _____v,  KC.NLCK,  KC.SLCK,  KC.PAUS,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,    _____v,
        _____v,  _____v,   _____v,     EURO,   _____v,   _____v,   _____v,       MU,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   ______,   GOCUS1,   GOCUS2,   _____v,    _____v,
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   ______,    GOSTD,   ______,  INFOKC1,    ______,
        _____v,  _____v,   ______,   _____v,   ______,   ______,   KC.SPC,   ______,   ______,   ______,  KC.RALT,  KC.RGUI,  KC.RCTL,     REDO,   _____v,   _____v,   _____v,    _____v, 
        _____v,  _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   _____v,   KC.INS,   _____v,   _____v,   _____v,    _____v,
    ],
]

# REWORK NEEDED : It kinda works but TRICKY as sometimes E or O passes thru combined with ALT and intercepted by current application menu shortcuts
combos.combos = [
    Chord((KC.Q, KC.E), KC.KUNIAE), #UNIAE),
    Chord((KC.O, KC.E), KC.KUNIOE), #UNIOE),
	#Sequence((KC.LEADER, KC.Q, KC.E), UNIAE),
	#Sequence((38, 40), UNIAE, timeout=500, match_coord=True),
]

if __name__ == '__main__':
    chti116.go()
    
